#include "mainwindow.h"
#include <fstream>

MainWindow::MainWindow() {
    inputField = new QLineEdit(this);
    firstTable = new QTableWidget(4, 2, this);
    firstTable->setHorizontalHeaderLabels(QStringList() << "Тип" << "Частина");
    firstTable->verticalHeader()->setVisible(false);
    firstTable->setEditTriggers(QAbstractItemView::NoEditTriggers);
    firstTable->setSelectionMode(QAbstractItemView::NoSelection); //заборона взаємодії з таблицею, нам це не потрібно
    firstTable->setFocusPolicy(Qt::NoFocus);
    firstTable->horizontalHeader()->setSectionResizeMode(0, QHeaderView::Stretch);
    firstTable->horizontalHeader()->setSectionResizeMode(1, QHeaderView::Fixed);
    firstTable->setColumnWidth(1, 80);
    auto addLeftItem = [this](int row, const QString &text) {
        QTableWidgetItem *item = new QTableWidgetItem(text);
        item->setTextAlignment(Qt::AlignCenter);
        firstTable->setItem(row, 0, item);
    };
    addLeftItem(0, "Кирилиця");
    addLeftItem(1, "Латиниця");
    addLeftItem(2, "Цифри");
    addLeftItem(3, "Інше");

    secondTable = new QTableWidget(10, 2, this);
    secondTable->setHorizontalHeaderLabels(QStringList() << "Число" << "Кількість");
    secondTable->verticalHeader()->setVisible(false);
    secondTable->setEditTriggers(QAbstractItemView::NoEditTriggers);
    secondTable->setSelectionMode(QAbstractItemView::NoSelection);
    secondTable->setFocusPolicy(Qt::NoFocus);
    secondTable->horizontalHeader()->setSectionResizeMode(0, QHeaderView::Stretch);
    secondTable->horizontalHeader()->setSectionResizeMode(1, QHeaderView::Fixed);
    secondTable->setColumnWidth(1, 80);
    for (int a = 0; a < 10; a++) {
        QTableWidgetItem *item = new QTableWidgetItem(QString::number(a));
        item->setTextAlignment(Qt::AlignCenter);
        secondTable->setItem(a, 0, item);
    }
    dominantScriptLabel = new QLabel();
    dominantScriptLabel->setAlignment(Qt::AlignCenter);
    replaceField1 = new QLineEdit();
    replaceField1->setMaxLength(1);
    
    replaceField2 = new QLineEdit();
    replaceField2->setMaxLength(1);
    QLabel *onLabel = new QLabel("на");
    QHBoxLayout *replaceLayout = new QHBoxLayout;
    QHBoxLayout *midLayout = new QHBoxLayout;
    QPushButton *replacebutton=new QPushButton("Замінити");
    QPushButton *maxbutton=new QPushButton("Найдовше");
    QPushButton *minbutton=new QPushButton("Найкоротше");
    QPushButton *sortbutton=new QPushButton("Власні назви");
    QPushButton *demobutton=new QPushButton("Перевірити конструктор копіювання");
    replaceLayout->addWidget(replacebutton);
    replaceLayout->addWidget(replaceField1);
    replaceLayout->addWidget(onLabel);
    replaceLayout->addWidget(replaceField2);
    replaceField1->setFixedWidth(30);
    replaceField2->setFixedWidth(30);
    replaceField1->setAlignment(Qt::AlignCenter);
    replaceField2->setAlignment(Qt::AlignCenter);
    connect(replacebutton, &QPushButton::clicked, this, &MainWindow::replace);
    connect(minbutton, &QPushButton::clicked, this, &MainWindow::shortest);
    connect(maxbutton, &QPushButton::clicked, this, &MainWindow::longest);
    connect(sortbutton, &QPushButton::clicked, this, &MainWindow::capSort);
    connect(demobutton, &QPushButton::clicked, this, &MainWindow::demo);
    replaceLayout->addSpacerItem(new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));
    //midLayout->addSpacerItem(new QSpacerItem(0, 20, QSizePolicy::Expanding, QSizePolicy::Minimum)); Якби нам не потрібно було розширювати таблицю
    midLayout->addWidget(demobutton);
    midLayout->addWidget(sortbutton);
    midLayout->addWidget(minbutton);
    midLayout->addWidget(maxbutton);
    replaceLayout->addWidget(new QLabel("Кількість речень в тексті:"));
    scounter=new QLabel();
    replaceLayout->addWidget(scounter);
    QHBoxLayout *tablesLayout = new QHBoxLayout;
    tablesLayout->addWidget(firstTable);
    tablesLayout->addSpacerItem(new QSpacerItem(20, 0, QSizePolicy::Expanding, QSizePolicy::Minimum));
    tablesLayout->addWidget(secondTable);
    QVBoxLayout *layout = new QVBoxLayout;
    layout->addWidget(inputField);
    layout->addLayout(replaceLayout);
    layout->addLayout(midLayout);
    layout->addLayout(tablesLayout);
    
    QPushButton *cyrToLatButton = new QPushButton("Кирилиця -> Латиниця");
    QPushButton *latToCyrButton = new QPushButton("Латиниця -> Кирилиця");
    connect(cyrToLatButton, &QPushButton::clicked, this, &MainWindow::replaceCyrillicToLatin);
    connect(latToCyrButton, &QPushButton::clicked, this, &MainWindow::replaceLatinToCyrillic);
    layout->addWidget(cyrToLatButton);
    layout->addWidget(latToCyrButton);
    layout->addWidget(dominantScriptLabel);
    
    QWidget *centralWidget = new QWidget(this);
    centralWidget->setLayout(layout);
    setCentralWidget(centralWidget);
    
    setMinimumSize(550, 450);
    
    saveAct = new QAction(tr("&Зберігти"), this); //Амперсанд вказує на гарячу клавішу
    saveAct->setShortcuts(QKeySequence::Save);
    connect(saveAct, &QAction::triggered, this, &MainWindow::save);
    
    loadAct = new QAction(tr("&Завантажити"), this);
    loadAct->setShortcuts(QKeySequence::Open);
    connect(loadAct, &QAction::triggered, this, &MainWindow::load);
    
    QAction* newAct = new QAction(tr("&Новий"), this);
    newAct->setShortcuts(QKeySequence::New);
    connect(newAct, &QAction::triggered, this, &MainWindow::clear);
    
    QAction* exitAct = new QAction(tr("&Вихід"), this);
    exitAct->setShortcuts(QKeySequence::Quit);
    connect(exitAct, &QAction::triggered, this, &QWidget::close);
    
    QAction* aboutAct = new QAction(tr("&Про програму"), this);
    aboutAct->setShortcuts(QKeySequence::HelpContents);
    connect(aboutAct, &QAction::triggered, this, &MainWindow::about);

    QAction* aboutQtAct = new QAction(tr("Про &Qt"), this);
    connect(aboutQtAct, &QAction::triggered, qApp, &QApplication::aboutQt);
    QMenu* fileMenu = menuBar()->addMenu(tr("&Файл"));
    fileMenu->addAction(saveAct);
    fileMenu->addAction(loadAct);
    fileMenu->addAction(newAct);
    fileMenu->addSeparator();
    fileMenu->addAction(exitAct);
    QMenu* helpMenu = menuBar()->addMenu(tr("&Допомога"));
    helpMenu->addAction(aboutAct);
    helpMenu->addAction(aboutQtAct);
    setWindowTitle(tr("kursa4"));
    connect(inputField, &QLineEdit::textChanged, this, &MainWindow::updateStats);
    realstatusLabel = new QLabel("Готово", this);
    realstatusLabel->setAlignment(Qt::AlignRight);
    QWidget *statusContainer = new QWidget(this);
    QHBoxLayout *statusLayout = new QHBoxLayout(statusContainer);
    statusLayout->addWidget(realstatusLabel);
    menuBar()->setCornerWidget(statusContainer, Qt::TopRightCorner);
    realstatusLabel->setFixedWidth(400);
    updateStats();
}

#ifndef QT_NO_CONTEXTMENU
void MainWindow::contextMenuEvent(QContextMenuEvent *event)
{
    QMenu menu(this);
    menu.addAction(saveAct);
    menu.addAction(loadAct);
    menu.exec(event->globalPos());
}
#endif

void MainWindow::save()
{
    QString fileName = QFileDialog::getSaveFileName(this, tr("Зберігти файл"), "", tr("Текстові файли (*.txt)"));
    if (fileName.isEmpty()) return;
    
    std::ofstream file(fileName.toStdString());
    if (!file.is_open()) {
        QMessageBox::warning(this, tr("Помилка"), tr("Не вдалося відкрити файл для запису"));
        return;  
    }
    file << inputField->text().toStdString();
    if (!inputField->text().endsWith("\n")) file << '\n';
    
    file.close();
    realstatusLabel->setText("Файл збережено");
}

void MainWindow::load() {
    QString fileName = QFileDialog::getOpenFileName(this, tr("Відкрити файл"), "", tr("Текстові файли (*.txt);;Усі файли (*)"));
    if (fileName.isEmpty())return;

    std::wifstream file(fileName.toStdString());
    if (!file.is_open()) {
        QMessageBox::warning(this, tr("Помилка"), tr("Не вдалося відкрити файл для читання"));
        return;
    }
    file.imbue(std::locale("")); // Ставимо системну локаль, щоб бодай щось прочитати. ifstream цього не потребує, але і адекватно Кне читає кирилицю
    text.reset();
    while (file)file>>text; // Це жахливо. Але цього вимагає завдання.
    file.close();
    read=1;
    updateStats();
    realstatusLabel->setText("Файл завантажено");
}

void MainWindow::clear() {
    inputField->clear();
    updateStats();
    realstatusLabel->setText("Готово");
}

void MainWindow::about(){QMessageBox::about(this, tr("Про курсову роботу"),tr("<b>kursa4</b> - це непоганий приклад курсової роботи<br>Зроблено Іваном."));}
void MainWindow::updateStats() {
    if (read){inputField->setText(QString::fromStdWString(text.towstring()));read=0;}
        else text.set(inputField->text().toStdWString());
    const std::array<float, 4> stats = text.statc();
    for (char a=0;a<4;a++) {
        QString percent = stats[a] > 0 ? QString::number(stats[a]*100, 'f', 1) + "%" : "-"; // Якщо в тексті немає символів даного типу, виводимо "-"
        QTableWidgetItem *item = new QTableWidgetItem(percent);
        firstTable->setItem(a, 1, item);
    }
    if (text.empty())dominantScriptLabel->clear();else{
    if (stats[0] != stats[1]) {
        if (stats[0] > stats[1]) dominantScriptLabel->setText("Текст переважно кириличний");
        else dominantScriptLabel->setText("Текст переважно латинський");
    } else dominantScriptLabel->setText("Текст має однакову кількість кириличних та латинських символів");
      }
    const std::array<unsigned short, 10> digitStats = text.statn();
    for (int i = 0; i < 10; i++) {
        QString count = QString::number(digitStats[i]);
        QTableWidgetItem *item = new QTableWidgetItem(count);
        secondTable->setItem(i, 1, item);
    }
    scounter->setText(QString::number(text.sentences()));
    realstatusLabel->setText("Готово");
}

void MainWindow::replaceCyrillicToLatin() {
  if(text.empty()){
      realstatusLabel->setText("Немає даних - нічого замінювати");
      return;
    }
    text.cyrtolat();
    inputField->setText(QString::fromStdWString(text.towstring()));
    realstatusLabel->setText("Кирилиця замінена на латиницю");
}

void MainWindow::replaceLatinToCyrillic() {
  if(text.empty()){
      realstatusLabel->setText("Немає даних - нічого замінювати");
      return;
    }
    text.lattocyr();
    inputField->setText(QString::fromStdWString(text.towstring()));
    realstatusLabel->setText("Латиниця замінена на кирилицю");
}
void MainWindow::replace(){
  if(text.empty()){
      realstatusLabel->setText("Немає даних - нічого замінювати");
      return;
    }text.swap(replaceField1->text().toStdWString()[0],replaceField2->text().toStdWString()[0]);inputField->setText(QString::fromStdWString(text.towstring()));replaceField1->clear();replaceField2->clear();}
void MainWindow::shortest(){
  if(text.empty()){
      realstatusLabel->setText("Немає даних - нічого виводити");
      return;
    }QMessageBox::information(this, tr("Найкоротше речення"),QString::fromStdWString(text.shortestSentence()));}
void MainWindow::longest(){
  if(text.empty()){
      realstatusLabel->setText("Немає даних - нічого виводити");
      return;
    }QMessageBox::information(this, tr("Найдовше речення"),QString::fromStdWString(text.longestSentence()));}
void MainWindow::capSort(){
  if(text.empty()){
      realstatusLabel->setText("Немає даних - нічого виводити");
      return;
    }
  QString out;
    std::
    #ifdef SORT
        vector
    #else
        set
    #endif
        <std::wstring>words=text.capSort();
        for (const std::wstring &word : words){
            out+=QString::fromStdWString(word);
            if(word!=*words.rbegin())out+='\n';
          }
    QMessageBox::information(this, tr("Відсортовані власні назви"),out);
                             realstatusLabel->setText("Готово");
}
void MainWindow::demo(){
    if (text.towstring().empty()){
        realstatusLabel->setText("Немає даних - неможливо перевірити конструктор копіювання");
        return;
    }
    textChecker copy(text);
    QString out="До:\t"+QString::fromStdWString(text.towstring())+"\nПісля:\t"+QString::fromStdWString(copy.towstring())+'\n';
    if (text==copy)out+="Конструктор копіювання працює правильно!";else out+="Конструктор копіювання працює неправильно!";
    QMessageBox::information(this, tr("Перевірка роботи конструктора копіювання"),out);
    realstatusLabel->setText("Готово");
}
